package jdomXpath;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;
import org.jdom2.Attribute;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.filter.Filters;
import org.jdom2.input.SAXBuilder;
import org.jdom2.output.Format;
import org.jdom2.output.XMLOutputter;
import org.jdom2.xpath.XPathExpression;
import org.jdom2.xpath.XPathFactory;

public class PartyJdomXpath {

    public static void main(String args[]) {
        try {
            //Erzeugen eines JDOM-Dokuments anhand der Datei party.xml
            SAXBuilder builder = new SAXBuilder();
            // Document aus org.jdom2
            Document xmlDocument = builder.build(new File("party.xml"));

            //Lesen des Wurzelelements des JDOM-Dokuments xmlDocument
            Element party = xmlDocument.getRootElement();

            // Wann fand die Party statt?
            System.out.println(party.getAttribute("datum").getValue());
            
            // Wie heißt der erste Gast?
            Element albert = party.getChild("gast");
            System.out.println(albert.getAttribute("name").getValue());

             // Alle Attribute von Albert auslesen
            List<Attribute> attributes = albert.getAttributes();
            for(Attribute attribute : attributes){
               System.out.println(attribute.getName() + " " + attribute.getValue());
            }
            
            // Was hat dieser am Anfang getrunken
            Element albertGetraenk = party.getChild("gast").getChild("getraenk");
            System.out.println(albertGetraenk.getText());
            // Das Ganze in einer Zeile
            System.out.println(party.getChild("gast").getChildText("getraenk"));

            // Ist der erste Gast noch nüchtern?
            System.out.println("nüchtern: " + albert.getChild("zustand").getAttribute("nuechtern").getBooleanValue());

            // Was hat Albert alles auf der Party getrunken?
            List<Element> albertGetraenke = albert.getChildren("getraenk");
            for (Element e : albertGetraenke) {
                System.out.println(e.getText());
            }

            // Das erste Getränk von Albert wurde falsch erfasst . Er trank Wasser
            albert.removeChild("getraenk");
            Element wasser = new Element("getraenk");
            wasser.addContent("Wasser");
            albert.addContent(wasser);

            // Zacharias Zottelig trinkt jetzt auch ein Glas Wein
            Element wein = new Element("getraenk");
            wein.addContent("Wein");
            List<Element> gaeste = party.getChildren("gast");
            for (Element gast : gaeste) {
                if (gast.getAttribute("name").getValue().equals("Zacharias Zottelig")) {
                    gast.addContent(wein);
                }
            }

            // neuen Gast erzeugen und hinzufügen. Dieser trinkt auch Wein
            Element max = new Element("gast");
            max.setAttribute("name", "Max Muster");
            max.addContent(new Element("getraenk").addContent("Wein"));
            party.addContent(max);

            // Was wurde auf der Party alles getrunken -> Suche mit XPath
            XPathFactory xpf = XPathFactory.instance();
            XPathExpression<Element> xPathExpression = xpf.compile("//getraenk", Filters.element());
            List<Element> getraenke = xPathExpression.evaluate(xmlDocument);

//            XPath rolePath1 = XPath.newInstance("//getraenk");
//            List<Element> getraenke = rolePath1.selectNodes(xmlDocument);
            // Alle Getränke ausgeben
            if (getraenke.isEmpty()) {
                System.out.println("keine Getränke");
            } else {
                System.out.println("Anzahl Getränke " + getraenke.size());
                for (Element getraenk : getraenke) {
                    System.out.println(getraenk.getTextTrim());
                }
            }

            // Wer hat alles Wein getrunken
            xPathExpression = xpf.compile("//getraenk[node()='Wein']/..", Filters.element());
            List<Element> weinTrinker = xPathExpression.evaluate(xmlDocument);
            System.out.println("Weintrinker");
            if (!weinTrinker.isEmpty()) {
                for (Element e : weinTrinker) {
                    System.out.println("Name: " + e.getAttribute("name").getValue());
                }
            }

            // Alle Betrunkenen nach Hause schicken
            xPathExpression = xpf.compile("//zustand[@nuechtern='false']/..", Filters.element());
            List<Element> betrunkene = xPathExpression.evaluate(xmlDocument);
            for (Element e : betrunkene) {
                System.out.println(e.getAttribute("name").getValue() + " geht nach Hause");
                party.removeContent(e);
            }
            XMLOutputter outp = new XMLOutputter();
            outp.setFormat(Format.getPrettyFormat());  // only for nicer formatting

            // Formatierte Ausgabe auf der Konsole
            outp.output(xmlDocument, System.out);

            // Formatierte Ausgabe in eine Datei
            outp.output(xmlDocument, new FileOutputStream(new File("party2.xml")));
        } catch (JDOMException | IOException ex) {
            ex.printStackTrace();
        }
    }
}
