// http://www.javacodegeeks.com/2013/05/parsing-xml-using-dom-sax-and-stax-parser-in-java.html#dom
package sax;

import fachklassen.Mitarbeiter;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

public class MitarbeiterSaxParser {

    public static void main(String[] args) throws Exception {
        SAXParserFactory parserFactory = SAXParserFactory.newInstance();
        SAXParser saxParser = parserFactory.newSAXParser();
        SAXHandler handler = new SAXHandler();
        saxParser.parse(new File("employees.xml"), handler);
        //    saxParser.parse(ClassLoader.getSystemResourceAsStream("xml/employees.xml"), handler);
    }
}

/**
 * The Handler for SAX Events.
 */
class SAXHandler extends DefaultHandler {

    private List<Mitarbeiter> empList = null;
    private Mitarbeiter currEmp = null;
    private String content = null;

    @Override
    public void startDocument() throws SAXException {
        // compiled code
    }

    @Override
    public void endDocument() throws SAXException {
        //Mitarbeiterliste aus der XML-Datei ausgeben
        for (Mitarbeiter emp : empList) {
            System.out.println(emp);
        }
    }

    @Override
    //Triggered when the start of tag is found.
    public void startElement(String uri, String localName,
            String elementName, Attributes attributes)
            throws SAXException {
        if (elementName.equals("employees")) {
            empList = new ArrayList<>();
        } else if (elementName.equals("employee")) {
            currEmp = new Mitarbeiter();
            currEmp.setId(Integer.parseInt(attributes.getValue("id")));
        }
    }
    
    @Override
    public void characters(char[] ch, int start, int length)
            throws SAXException {
        content = String.copyValueOf(ch, start, length).trim();
    }

    
    @Override
    public void endElement(String uri, String localName,
            String elementName) throws SAXException {
        if (elementName.equals("employee")) {
            // Füge den neuen Mitarbeiter zur Liste hinzu
            empList.add(currEmp);
            // Alle anderen Tags dienen zum Setzen der Attribute
        } else if (elementName.equals("age")) {
            currEmp.setAlter(Integer.parseInt(content));
        } else if (elementName.equals("name")) {
            currEmp.setName(content);
        } else if (elementName.equals("gender")) {
            currEmp.setGeschlecht(content);
        } else if (elementName.equals("role")) {
            currEmp.setFunktion(content);
        }
    }
}
